@@ -38,7 +38,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)collect.c	2.44 (gritter) 1/10/05";
+static char sccsid[] = "@(#)collect.c	2.50 (gritter) 6/9/05";
 #endif
 #endif /* not lint */
 
@@ -168,10 +168,13 @@ print_collf(FILE *collf, struct header *hp)
 			maxlines--;
 		if (hp->h_bcc)
 			maxlines--;
-		if (hp->h_replyto)
-			maxlines--;
 		if (hp->h_attach)
 			maxlines--;
+		maxlines -= myaddrs() != NULL || hp->h_from != NULL;
+		maxlines -= value("ORGANIZATION") != NULL ||
+			hp->h_organization != NULL;
+		maxlines -= value("replyto") != NULL || hp->h_replyto != NULL;
+		maxlines -= value("sender") != NULL || hp->h_sender != NULL;
 		if (linecnt > maxlines) {
 			cp = get_pager();
 			if (sigsetjmp(pipejmp, 1))
@@ -186,7 +189,7 @@ print_collf(FILE *collf, struct header *hp)
 	}
 	fprintf(obuf, catgets(catd, CATSET, 62,
 				"-------\nMessage contains:\n"));
-	gf = GTO|GSUBJECT|GCC|GBCC|GNL|GREPLYTO|GFILES;
+	gf = GIDENT|GTO|GSUBJECT|GCC|GBCC|GNL|GFILES;
 	if (value("fullnames"))
 		gf |= GCOMMA;
 	puthead(hp, obuf, gf, SEND_TODISP, CONV_NONE, NULL, NULL);
@@ -643,7 +646,8 @@ cont:
 		    (value("dot") != NULL || value("ignoreeof") != NULL))
 			break;
 		if (linebuf[0] != escape || (value("interactive") == NULL &&
-					tildeflag <= 0)) {
+					tildeflag == 0 ||
+					tildeflag < 0)) {
 			if (putline(collf, linebuf, count) < 0)
 				goto err;
 			continue;
@@ -719,6 +723,14 @@ cont:
 						value("bsdorder") != NULL);
 			while (hp->h_to == NULL);
 			goto cont;
+		case 'H':
+			/*
+			 * Grab extra headers.
+			 */
+			do
+				grabh(hp, GEXTRA, 0);
+			while (check_from_and_sender(hp->h_from, hp->h_sender));
+			goto cont;
 		case 't':
 			/*
 			 * Add to the To list.
@@ -819,7 +831,8 @@ cont:
 			/*
 			 * Insert the contents of a signature variable.
 			 */
-			if ((cp = value(c == 'a' ? "sign" : "Sign")) != NULL) {
+			if ((cp = value(c == 'a' ? "sign" : "Sign")) != NULL &&
+					*cp != '\0') {
 				if (is_a_tty[0])
 					putesc(cp, stdout);
 				putesc(cp, collf);
@@ -981,6 +994,8 @@ makeheader(FILE *fp, struct header *hp)
 		Fclose(collf);
 	Fclose(fp);
 	collf = nf;
+	if (check_from_and_sender(hp->h_from, hp->h_sender))
+		return STOP;
 	return OKAY;
 }
 
@@ -1177,7 +1192,7 @@ void
 savedeadletter(FILE *fp)
 {
 	FILE *dbuf;
-	int c;
+	int c, lines = 0, bytes = 0;
 	char *cp;
 
 	if (fsize(fp) == 0)
@@ -1188,9 +1203,15 @@ savedeadletter(FILE *fp)
 	umask(c);
 	if (dbuf == NULL)
 		return;
-	while ((c = getc(fp)) != EOF)
+	printf("\"%s\" ", cp);
+	while ((c = getc(fp)) != EOF) {
 		putc(c, dbuf);
+		bytes++;
+		if (c == '\n')
+			lines++;
+	}
 	Fclose(dbuf);
+	printf("%d/%d\n", lines, bytes);
 	rewind(fp);
 }
 
