@@ -33,7 +33,7 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)nss.c	1.34 (gritter) 11/17/04";
+static char sccsid[] = "@(#)nss.c	1.38 (gritter) 6/9/05";
 #endif
 #endif /* not lint */
 
@@ -68,6 +68,8 @@ static sigjmp_buf	nssjmp;
 
 #include "extern.h"
 
+#include "nsserr.c"
+
 static char *password_cb(PK11SlotInfo *slot, PRBool retry, void *arg);
 static SECStatus bad_cert_cb(void *arg, PRFileDesc *fd);
 static const char *bad_cert_str(void);
@@ -117,80 +119,9 @@ static const char *
 bad_cert_str(void)
 {
 	int	ec;
-	char	*es, eb[40];
 
 	ec = PORT_GetError();
-	switch (ec) {
-	case SEC_ERROR_INVALID_AVA:
-		es = "Invalid AVA";
-		break;
-	case SEC_ERROR_INVALID_TIME:
-		es = "Invalid time";
-		break;
-	case SEC_ERROR_BAD_SIGNATURE:
-		es = "Bad signature";
-		break;
-	case SEC_ERROR_EXPIRED_CERTIFICATE:
-		es = "Certificate expired";
-		break;
-	case SEC_ERROR_UNKNOWN_ISSUER:
-		es = "Unknown issuer";
-		break;
-	case SEC_ERROR_UNTRUSTED_CERT:
-		es = "Untrusted certificate";
-		break;
-	case SEC_ERROR_CERT_VALID:
-		return SECSuccess;
-	case SEC_ERROR_EXPIRED_ISSUER_CERTIFICATE:
-		es = "Issuer certificate expired";
-		break;
-	case SEC_ERROR_CRL_EXPIRED:
-		es = "CRL expired";
-		break;
-	case SEC_ERROR_CRL_BAD_SIGNATURE:
-		es = "Bad CRL signature";
-		break;
-	case SEC_ERROR_EXTENSION_VALUE_INVALID:
-		es = "Extension value invalid";
-		break;
-	case SEC_ERROR_CERT_USAGES_INVALID:
-		es = "Invalid certificate usage";
-		break;
-	case SEC_ERROR_UNKNOWN_CRITICAL_EXTENSION:
-		es = "Unknown critical extension in certificate";
-		break;
-	case SEC_ERROR_IO:
-		es = "I/O error";
-		break;
-	case SEC_ERROR_LIBRARY_FAILURE:
-		es = "Library failure";
-		break;
-	case SEC_ERROR_BAD_DATA:
-		es = "Bad data";
-		break;
-	case SEC_ERROR_REVOKED_CERTIFICATE:
-		es = "Revoked certificate";
-		break;
-	case SEC_ERROR_BAD_DATABASE:
-		es = "Bad database";
-		break;
-	case SEC_ERROR_UNTRUSTED_ISSUER:
-		es = "Untrusted issuer";
-		break;
-	case SEC_ERROR_DUPLICATE_CERT:
-		es = "Duplicate certificate";
-		break;
-	case SEC_ERROR_DUPLICATE_CERT_NAME:
-		es = "Duplicate certificate name";
-		break;
-	case SEC_ERROR_CA_CERT_INVALID:
-		es = "CA certificate invalid";
-		break;
-	default:
-		snprintf(eb, sizeof eb, "Unknown error %d", ec);
-		es = eb;
-	}
-	return es;
+	return nss_strerror(ec);
 }
 
 static enum okay
@@ -315,14 +246,13 @@ nss_gen_err(const char *fmt, ...)
 	va_start(ap, fmt);
 	vfprintf(stderr, fmt, ap);
 	va_end(ap);
-	PR_GetError();
 	if ((len = PR_GetErrorTextLength()) > 0) {
 		text = ac_alloc(len);
 		if (PR_GetErrorText(text) > 0)
-			fprintf(stderr, ": %s", text);
+			fprintf(stderr, ": %s\n", text);
 		ac_free(text);
-	}
-	fputc('\n', stderr);
+	} else
+		fprintf(stderr, ": %s.\n", nss_strerror(PR_GetError()));
 }
 
 FILE *
@@ -335,10 +265,13 @@ smime_sign(FILE *ip)
 	CERTCertificate	*cert;
 	CERTCertDBHandle	*handle;
 	FILE	*hp, *bp, *sp;
+	char	*addr;
 
 	if (nss_init() != OKAY)
 		return NULL;
-	if ((cert = get_signer_cert(myaddr())) == NULL)
+	if ((addr = myorigin()) == NULL)
+		return NULL;
+	if ((cert = get_signer_cert(addr)) == NULL)
 		return NULL;
 	handle = CERT_GetDefaultCertDB();
 	if ((msg = NSS_CMSMessage_Create(NULL)) == NULL) {
@@ -760,12 +693,11 @@ verify1(struct message *m, int n)
 	int	nlevels, i;
 	int	status = 0;
 	int	foundsender = 0;
-	char	*from;
+	char	*sender;
 
 	if ((m = getsig(m, n, &msg)) == NULL)
 		return 1;
-	if ((from = hfield("from", m)) != NULL)
-		from = skin(from);
+	sender = getsender(m);
 	handle = CERT_GetDefaultCertDB();
 	nlevels = NSS_CMSMessage_ContentLevelCount(msg);
 	for (i = 0; i < nlevels; i++) {
@@ -844,13 +776,13 @@ verify1(struct message *m, int n)
 				svs = NSS_CMSUtil_VerificationStatusToString
 					(vs);
 				addr = CERT_GetCertEmailAddress(&cert->subject);
-				if (from != NULL && addr != NULL &&
-							!asccasecmp(from, addr))
-						foundsender++;
+				if (sender != NULL && addr != NULL &&
+						asccasecmp(sender, addr) == 0)
+					foundsender++;
 				else {
 					addr = CERT_GetFirstEmailAddress(cert);
-					while (from && addr) {
-						if (!asccasecmp(from, addr)) {
+					while (sender && addr) {
+						if (!asccasecmp(sender, addr)) {
 							foundsender++;
 							break;
 						}
@@ -885,10 +817,10 @@ verify1(struct message *m, int n)
 		}
 	}
 	if (foundsender == 0) {
-		if (from) {
+		if (sender) {
 			fprintf(stderr, "Signers of message "
 					"%d do not include the sender <%s>\n",
-				n, from);
+				n, sender);
 			status = -1;
 		} else
 			fprintf(stderr, "Warning: Message %d has no From: "
